-------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
-------------------------------------------

local LrApplication = import 'LrApplication'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrShell = import 'LrShell'
local LrTasks = import 'LrTasks'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 

local myLogger = LrLogger('PhotomatixLogger')
myLogger:enable("print") 

require 'CreateHdrUtils'

CreateHdrUploadTask = {}

function CreateHdrUploadTask.outputToLog(message)
	myLogger:trace(message)
end

function CreateHdrUploadTask.processRenderedPhotos( functionContext, exportContext )
	local pmdebug = true
	
	if pmdebug then
		CreateHdrUploadTask.outputToLog('Start CreateHdrUploadTask.processRenderedPhotos')
	end

	
	
	local lrVersion=LrApplication.versionTable
	if lrVersion then
		prefs.LrVersion=lrVersion().major
	else
		prefs.LrVersion=1
	end

-------------------------- Showing plugin dialog and getting the options selected ------------------------------------	
	require 'CreateHdrSettings'

	CreateHdrSettings.getOptions()

 	if prefs.processType=='none' then
		return
	end
	

-------------------------- Getting path to command line program (PhotomatixCL for WIN_ENV, MergeToHDR for MAC_ENV)  ------------------------------------	
	local appPath=LrPathUtils.child(_PLUGIN.path, "app")

	local clProgName="PhotomatixCL"
	if (MAC_ENV) then
		clProgName="MergeToHDR"
	end

	appPath=LrPathUtils.child(appPath, clProgName)
	
	if (WIN_ENV) then
		appPath=LrPathUtils.addExtension(appPath, "exe")
	end

	if pmdebug then
		CreateHdrUploadTask.outputToLog(string.format('appPath: %s', appPath))
	end

	if not LrFileUtils.exists(appPath) then
		CreateHdrUploadTask.outputToLog(string.format('Could not find command line program at: %s',appPath))
		LrErrors.throwUserError(string.format("Could not find %s program in %s",clProgName, LrPathUtils.parent(appPath))) 		
		return
	end


-------------------------- Checking license key and showing registration dialog if necessary  ------------------------------------	

	local enteredKeyOK=false
	local hasEnteredKey=false
--[[
	local regOK=false
	if ((prefs.isRegistered~=nil) and (prefs.licenseKey~=nil)) then
		if (prefs.isRegistered) then
			regOK=true
		end
	end

	if not regOK then
--]]	
	local hasLicense=CreateHdrUtils.Validate("")

	if not hasLicense then

		require 'CreateHdrRegistration'
		local retRegDialog=CreateHdrRegistration.showRegDialog()
			
		if ( (retRegDialog~=nil) and (retRegDialog=='cancel') ) then
			return
		end

		if ((prefs.enteredKey~=nil) and (prefs.enteredKey~="")) then
			hasEnteredKey=true
			enteredKeyOK=CreateHdrUtils.Validate(prefs.enteredKey)
			if enteredKeyOK then
				prefs.isRegistered=true
				prefs.licenseKey=prefs.enteredKey
			else	
				prefs.enteredKey=""
			end
		end
	end
--	end
	
	if hasEnteredKey then
		if enteredKeyOK then
			dialogMessageOK=LOC "$$$/Photomatix/Registration/RegistrationMessageOK=The license key you entered is valid."		
			dialogInfoOK=LOC "$$$/Photomatix/Registration/RegistrationInfoOK=Thank you for registering your copy of the 'Merge to 32-bit HDR' plug-in. There won't be any watermark added to the merged 32-bit file."		
			LrDialogs.message(dialogMessageOK, dialogInfoOK)
		else
			dialogMessageFailed=LOC "$$$/Photomatix/Registration/RegistrationMessageFailed=Registration did not go through"		
			dialogInfoFailed=LOC "$$$/Photomatix/Registration/RegistrationInfoFailed=The license key you entered could not be validated. A watermark will be added to the merged 32-bit file."		
			local userActionConf = LrDialogs.confirm(dialogMessageFailed, dialogInfoFailed, LOC "$$$/Photomatix/General/Continue=Continue")
			if ( (userActionConf~=nil) and (userActionConf=='cancel') ) then
				return
			end
		end
	end

	
	local txtProgress=LOC "$$$/Photomatix/Progress/ExportPhotos=Exporting photos..."
	local nFiles=0	
	local retour=-1
	local tempPath = nil
	local photoToStackWith=nil
	local pathPhotoToStackWith=nil
	local outputTempFile
	local photosProcessed=false

	LrFunctionContext.callWithContext('ExecuteCreateHDR', function(context)

		local outerProgressScope = LrDialogs.showModalProgressDialog({
					title = LOC "$$$/Photomatix/General/Create32HDR=Merge to 32-bit HDR",
					caption = txtProgress,
					cannotCancel = true,
					functionContext = context,
				})

		outerProgressScope:setIndeterminate()

-------------------------- Setting path to export to in temporary directory  ------------------------------------	
	local sysTempPath = LrPathUtils.getStandardFilePath('temp')
	repeat
		local now = LrDate.currentTime()
		tempPath = LrPathUtils.child(sysTempPath, LrDate.timeToUserFormat(now, 'temp_lrexport2photomatix_%m%d%H%M%S'))
		if LrFileUtils.exists(tempPath) then
			tempPath = nil
		else
			LrFileUtils.createAllDirectories(tempPath)
		end
	until tempPath
	
	if pmdebug then 
		CreateHdrUploadTask.outputToLog(string.format('tempPath: %s',tempPath))
	end

	local exportSession = exportContext.exportSession
	
	local activeCatalog = LrApplication.activeCatalog()

    -- we retrieve the photo to stack with now so that LR still stacks with the originally selected photo in case the user selected another photo before Photomatix reimports back the resulting image
	if (prefs.stackWith) then
		if prefs.LrVersion >= 3 then
			activeCatalog:withReadAccessDo( function()
			photoToStackWith = activeCatalog:getTargetPhoto()	
			end )
			pathPhotoToStackWith=photoToStackWith:getRawMetadata('path')
		else
			activeCatalog:withReadAccessDo( function()
			photoToStackWith = activeCatalog.targetPhoto	
			pathPhotoToStackWith=photoToStackWith.path
			end )
		end
		if pmdebug then
			CreateHdrUploadTask.outputToLog(string.format('path of photo to stack with: %s', pathPhotoToStackWith))
		end
	end

	
	
	local exportParams = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()

	local progressScope = exportContext:configureProgress{
						title = LOC( "$$$/Photomatix/Upload/ProgressTransfer=Transferring ^1 photos to Photomatix Pro for HDR merge", nPhotos ),
					}

	local files = {}
	nFiles=0
	local inputFilesArgs=""
	
-------------------------- Exporting photos to temporary directory ------------------------------------

	for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
	
		local success, pathOrMessage = rendition:waitForRender()
		
		if progressScope:isCanceled() then break end
		
		if success then
			local copyPath = LrPathUtils.child(tempPath, LrPathUtils.leafName(pathOrMessage))
			LrFileUtils.copy(pathOrMessage, copyPath)
			table.insert(files, copyPath)
			inputFilesArgs=string.format('%s \"%s\"',inputFilesArgs,copyPath)
			nFiles = nFiles + 1
		else
			LrErrors.throwUserError("Lightroom rendition failed. Please check that the selected photos exist.")
			---	error(pathOrMessage)
		end 
	end

	if nFiles > 1 then
		local doesExist=LrFileUtils.exists(appPath) 

		if doesExist then
		
-------------------------- Building command line arguments to pass to PhotomatixClient ------------------------------------
		
			local ldrBits=32
			if prefs.outputFormat=='tif16' then
				ldrBits=16
			end

			local PhotomatixClientCommand
			if WIN_ENV then
				local outFile = prefs.outputPath
				if string.match(outFile, ".tif$") ~= nil then
					outFile = string.sub(outFile, 1, string.len(outFile)-4)
				end
				PhotomatixClientCommand=string.format('\"%s\" -z -3 -oo \"%s\"', appPath, outFile)
				if ldrBits == 16 then
					PhotomatixClientCommand=string.format('%s -h tif16', PhotomatixClientCommand)  -- NOTE: Not currently implemented in PhotomatixCL
				else
					PhotomatixClientCommand=string.format('%s -h tif32', PhotomatixClientCommand)
				end
				if prefs.chkAlign then
					local alignType = prefs.alignType
					if alignType == "2" and (not prefs.chkPerspective) then
						alignType = "3"
					end
					
					if prefs.chkCrop then
						PhotomatixClientCommand=string.format('%s -a%s', PhotomatixClientCommand, alignType)
					else
						PhotomatixClientCommand=string.format('%s -a%sn', PhotomatixClientCommand, alignType)
					end
				end
			else
			
				PhotomatixClientCommand=string.format('\"%s\" -o \"%s\"', appPath, prefs.outputPath)
				PhotomatixClientCommand=string.format('%s -bi %d', PhotomatixClientCommand, ldrBits)
				if prefs.chkAlign then
					if prefs.chkCrop then
						PhotomatixClientCommand=string.format('%s -a%sc', PhotomatixClientCommand, prefs.alignType)
					else
						PhotomatixClientCommand=string.format('%s -a%sn', PhotomatixClientCommand, prefs.alignType)
					end
					if prefs.chkPerspective then
						PhotomatixClientCommand=string.format('%sp', PhotomatixClientCommand)
					else
						PhotomatixClientCommand=string.format('%sn', PhotomatixClientCommand)
					end
				end
			
			end

			if prefs.chkGhosting then
				PhotomatixClientCommand=string.format('%s -g2n', PhotomatixClientCommand)
			end

			if prefs.chkReduceNoise then
				PhotomatixClientCommand=string.format('%s -no%s', PhotomatixClientCommand, prefs.nrWhat)			
			end
			
			if prefs.needNormalise then
				PhotomatixClientCommand=string.format('%s -hn', PhotomatixClientCommand)							
			end
			
			if MAC_ENV then	-- we add the path of the folder containing the exported files to the arguments passed to PhotomatixClient
				PhotomatixClientCommand=string.format('%s -f %s',PhotomatixClientCommand, tempPath)			
			end

			outputTempFile=string.format('%sCmdOutput', tempPath)
			if (WIN_ENV) then
				PhotomatixClientCommand=string.format('%s %s > \"%s\"', PhotomatixClientCommand, inputFilesArgs, outputTempFile)		
				PhotomatixClientCommand=string.format('\"%s\"', PhotomatixClientCommand)
			else
				PhotomatixClientCommand=string.format('%s %s %s', PhotomatixClientCommand, prefs.processType, inputFilesArgs)		
			end

			progressScope:done()
				
			CreateHdrUploadTask.outputToLog(string.format('PhotomatixClientCommand: %s', PhotomatixClientCommand))

			if (WIN_ENV) then 
				LrFileUtils.delete(outputTempFile) -- in case it is there from before
			end
			
-------------------------- Calling command-line program ------------------------------------


			local txtProgress=LOC "$$$/Photomatix/Progress/AlignAndMerge=Aligning and merging to HDR..."
			if prefs.chkReduceNoise then
				if prefs.chkAlign then
					txtProgress=LOC "$$$/Photomatix/Progress/AlignDenoiseAndMerge=Aligning, denoising and merging to HDR..."
				else
					txtProgress=LOC "$$$/Photomatix/Progress/DenoiseAndMerge=Denoising and merging to HDR..."
				end
			else
				if not prefs.chkAlign then
					txtProgress=LOC "$$$/Photomatix/Progress/Merge=Merging to HDR..."
				end		
			end
			
			outerProgressScope:setCaption(txtProgress)
			
--			LrFunctionContext.callWithContext('ExecuteCreateHDR', function(context)

--				local progressScope = LrDialogs.showModalProgressDialog({
--					title = LOC "$$$/Photomatix/General/Create32HDR=Merge to 32-bit HDR",
--					caption = txtProgress,
--					cannotCancel = true,
--					functionContext = context,
--				})

--				progressScope:setIndeterminate()

				retour=LrTasks.execute(PhotomatixClientCommand)
				photosProcessed = true
			
				outerProgressScope:done()

			end
		end
	end)  -- End of section done while showing progress dialog
	
	if photosProcessed then
			CreateHdrUploadTask.outputToLog(string.format('after call to app - prefs.MergeIsRunning: %d',prefs.MergeIsRunning))
			
			if pmdebug then
				CreateHdrUploadTask.outputToLog(string.format('retour PhotomatixClient: %d', retour))
			end
			
			-- Remove temp files, because PhotomatixCL does not do this.
			if WIN_ENV then
				if string.find(tempPath, "lrexport2photomatix") ~= nil then  -- Make extra-sure that this really is a temp path
					LrFileUtils.delete(tempPath)
				end
			end
-------------------------- Returning from command-line program ------------------------------------

			local processingSucceeded = true
			if WIN_ENV then
				local outputMessage
				if math.abs(retour) >= 0x40000000 and math.abs(retour) <= 0xff000000 and WIN_ENV then
					-- These seem to correspond to PhotomatiCL crashing on Windows, so we are not likely to get a sensible output message
					outputMessage = "Processing failed"
					processingSucceeded = false
				elseif retour <= 0 or retour > 0xff000000 then
					processingSucceeded = false
					-- PhotomatixCL finished, but returned an error code
					local handle = io.open(outputTempFile, 'rb')
					if handle then
						CreateHdrUploadTask.outputToLog("Handle ok")
						while true do
							local currLine = handle:read('*line')
							if currLine == nil then break end
							outputMessage = currLine
						end
           				handle:close()
					end
				end
				
				if outputMessage == nil or string.find(outputMessage, "^%d+$") then  -- If just a number, there was no final message, and we just got one of the normal codes printed by PhotomatixCL
					outputMessage = ""
					if retour == 0xfffffffc then  -- -4: out of memory, and no custom error message was displayed
						outputMessage=LOC "$$$/Photomatix/General/Error/OutOfMemory=There was insufficient memory to process your images"
					end
				end
				
				local formattedMsg = string.format('%s',outputMessage)
				CreateHdrUploadTask.outputToLog(string.format('%s',outputMessage))

				if not processingSucceeded then
					LrDialogs.message("Error while exporting: " .. outputMessage, nil, "critical")
				end
			else
				-- MAC_ENV
				if retour ~= 0 then processingSucceeded = false end
			end

			if processingSucceeded then
				activeCatalog = LrApplication.activeCatalog()
				local undoActionName=string.format('Add %s to catalog', LrPathUtils.leafName(prefs.outputPath))

				if LrFileUtils.exists(prefs.outputPath)	then
					CreateHdrUploadTask.outputToLog(string.format('reimporting file: %s',prefs.outputPath))
				else
					CreateHdrUploadTask.outputToLog(string.format('file to reimport does not exist: %s',prefs.outputPath))
					LrDialogs.message("Could not access file to re-import", prefs.outputPath, "info")
				end
				
				if (prefs.stackWith) then
					photoToStackWith=nil
					activeCatalog:withReadAccessDo( function()
						photoToStackWith = activeCatalog:findPhotoByPath(pathPhotoToStackWith)
					end )
				end
				
				-- See if the photo we are importing is already in the catalog
				local existingPhoto = nil
				activeCatalog:withReadAccessDo( function()
				    existingPhoto = activeCatalog:findPhotoByPath(prefs.outputPath)
				end )
				
	
				local addedPhoto=nil
				local status, err
				if (photoToStackWith==nil) then
					activeCatalog:withWriteAccessDo( undoActionName, function()
						-- Work around bug in LR3 that raises an error when adding a photo that already exists
						status, err = LrTasks.pcall ( function()
							addedPhoto=activeCatalog:addPhoto(prefs.outputPath)
						end )
						
						addedPhoto:setPropertyForPlugin( _PLUGIN, 'software_tag', "Photomatix Merge To 32-bit HDR" )

					end )
				else
					activeCatalog:withWriteAccessDo( undoActionName, function()
						-- Work around bug in LR3 that raises an error when adding a photo that already exists
						status, err = LrTasks.pcall ( function()
							activeCatalog:addPhoto(prefs.outputPath, photoToStackWith, 'below')
						end )
						addedPhoto:setPropertyForPlugin( _PLUGIN, 'software_tag', "Photomatix Merge To 32-bit HDR" )
					end )				
				end
				
				
				-- If importing into 
				if not status then
					if existingPhoto == nil then
						LrDialogs.message("Error while importing photo:",  err, "critical")
					-- else ignore error
					end
				end
			end

			if (WIN_ENV) then 
				LrFileUtils.delete(outputTempFile)
			end	
	end
end

